@@ -40,7 +40,7 @@
 
    -----
 
-   canon_pp.c: $Revision: 1.2 $
+   canon_pp.c: $Revision: 1.10 $
 
    This file is part of the canon_pp backend, supporting Canon FBX30P 
    and NX40P scanners
@@ -50,19 +50,21 @@
 #include  <lalloca.h>		/* MUST come first for AIX! */
 #endif
 
-#define VERSION "$Revision: 1.2 $"
+#define VERSION "$Revision: 1.10 $"
 #define BACKEND_NAME canon_pp
 
 #define THREE_BITS 0xE0
 #define TWO_BITS 0xC0
+#define MM_PER_IN 25.4
 
 #include  <string.h>
+#include  <math.h>
 #include  <unistd.h>
 #include  <sys/stat.h>
 #include  <sys/types.h>
 #include  <stdlib.h>
 #include  <errno.h>
-#include <ieee1284.h>
+#include  <ieee1284.h>
 
 #include  "../include/sane/sane.h"
 #include  "../include/sane/saneopts.h"
@@ -80,7 +82,6 @@
 /* Prototypes */
 static SANE_Status init_device(struct parport *pp);
 
-static void update_ranges(CANONP_Scanner *cs);
 /* create a calibration file and give it initial values */
 static int init_cal(char *file);
 
@@ -110,8 +111,8 @@ static const SANE_String_Const cmodes[] = { "Greyscale", "Colour", NULL };
 /* bit depths */
 static const SANE_String_Const depths[] = { "8", "12", NULL };
 /* resolutions */
-static const SANE_Int res330[] = {3, 75, 150, 300};
-static const SANE_Int res630[] = {4, 75, 150, 300, 600};
+static const SANE_Int res300[] = {3, 75, 150, 300};
+static const SANE_Int res600[] = {4, 75, 150, 300, 600};
 
 
 /*************************************************************************
@@ -127,9 +128,10 @@ static const SANE_Int res630[] = {4, 75, 150, 300, 600};
 sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 {
 	SANE_Status status = SANE_STATUS_GOOD;
-	int i, tmp;
+	int i, tmp; 
+	int tmp_im = INITMODE_AUTO;
 	FILE *fp;
-	char line[80];
+	char line[81]; /* plus 1 for a null */
 	char *tmp_wf, *tmp_port;
 	CANONP_Scanner *s_tmp;
 
@@ -175,13 +177,16 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 	if (num_devices == 0)
 		status = SANE_STATUS_IO_ERROR;
 
+	/* just to be extra sure, the line will always have an end: */
+	line[sizeof(line)-1] = '\0';
+
 	/* 
 	 * Read information from config file: pixel weight location and default 
 	 * port.
 	 */
 	if((fp = sanei_config_open(CANONP_CONFIG_FILE)))
 	{
-		while(sanei_config_read(line, sizeof (line), fp))
+		while(sanei_config_read(line, sizeof (line) - 1, fp))
 		{
 			DBG(100, "sane_init: >%s<\n", line);
 			if(line[0] == '#')	/* ignore line comments */
@@ -226,19 +231,16 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 								tmp_port))
 					{
 						s_tmp->weights_file = tmp_wf;
-						break;
-						DBG(100, "sane_init: "
-								"Successfully"
-								" parsed "
+						DBG(100, "sane_init: Parsed "
 								"cal.\n");
+						break;
 					}
 					s_tmp = s_tmp->next;
 				}
 				if (s_tmp == NULL)
 				{
-					/* we made it all the way
-					 * through the list and didn't
-					 * find the port */
+					/* we made it all the way through the
+					 * list and didn't find the port */
 					free(tmp_wf);
 					DBG(10, "sane_init: calibrate line is "
 							"for unknown port!\n");
@@ -263,6 +265,57 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 				force_nibble = SANE_TRUE;
 				continue;
 			}
+
+			if(strncmp(line,"init_mode ", 10) == 0)
+			{
+
+				/* parse what sort of initialisation mode to 
+				 * use */
+				if (strncmp(line+10, "FB620P", 6) == 0)
+					tmp_im = INITMODE_20P;
+				else if (strncmp(line+10, "FB630P", 6) == 0)
+					tmp_im = INITMODE_30P;
+				else if (strncmp(line+10, "AUTO", 4) == 0)
+					tmp_im = INITMODE_AUTO;
+
+				/* now work out which port it blongs to */
+
+				tmp_port = strstr(line+10, " ") + 1;
+
+				if (tmp_port == NULL)
+				{
+					/* first_dev should never be null here
+					 * because we found at least one 
+					 * parallel port above */
+					first_dev->init_mode = tmp_im;
+					DBG(100, "sane_init: Parsed init-1.\n");
+					continue;
+				}
+
+
+				s_tmp = first_dev;
+				while (s_tmp != NULL)
+				{
+					if (!strcmp(s_tmp->params.port->name,
+								tmp_port+1))
+					{
+						s_tmp->init_mode = tmp_im;
+						DBG(100, "sane_init: Parsed "
+								"init.\n");
+						break;
+					}
+					s_tmp = s_tmp->next;
+				}
+				if (s_tmp == NULL)
+				{
+					/* we made it all the way through the
+					 * list and didn't find the port */
+					DBG(10, "sane_init: init_mode line is "
+							"for unknown port!\n");
+				}
+
+				continue;
+			}
 			DBG(1, "sane_init: Unknown configuration command!");
 
 		} 
@@ -292,7 +345,8 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 		 * us to call ieee1284_close in any of the remaining error
 		 * cases in this loop. */
 #if 0
-		tmp = sanei_canon_pp_detect(s_tmp->params.port);
+		tmp = sanei_canon_pp_detect(s_tmp->params.port, 
+				s_tmp->init_mode);
 
 
 		if (tmp && (s_tmp->ieee1284_mode != M1284_NIBBLE))
@@ -303,7 +357,8 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 
 			s_tmp->ieee1284_mode = M1284_NIBBLE;
 			sanei_canon_pp_set_ieee1284_mode(s_tmp->ieee1284_mode);
-			tmp = sanei_canon_pp_detect(s_tmp->params.port);
+			tmp = sanei_canon_pp_detect(s_tmp->params.port, 
+					s_tmp->init_mode);
 		}
 		/* still no go? */
 		if (tmp)
@@ -326,46 +381,41 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 		}
 		
 		DBG(2, "sane_init: >> initialise\n");
-		tmp = sanei_canon_pp_initialise(&(s_tmp->params));
+		tmp = sanei_canon_pp_initialise(&(s_tmp->params), 
+				s_tmp->init_mode);
 		DBG(2, "sane_init: << %d initialise\n", tmp);
-		/* put it back to sleep until we're ready to 
-		 * open for business again  */
-		sanei_canon_pp_sleep_scanner(s_tmp->params.port);
-		/* leave the port open but not claimed - this is regardless 
-		 * of the return value of initialise */
-		ieee1284_release(s_tmp->params.port);
-		if (tmp && (s_tmp->ieee1284_mode != M1284_NIBBLE))
-		{
-			/* A failure, try again in nibble mode... */
-			DBG(1, "sane_init: Failed on ECP mode, falling "
-					"back to nibble mode\n");
-
-			s_tmp->ieee1284_mode = M1284_NIBBLE;
-			sanei_canon_pp_set_ieee1284_mode(s_tmp->ieee1284_mode);
-			tmp = sanei_canon_pp_initialise(&(s_tmp->params));
-		}
 		if (tmp) {
 			DBG(10, "sane_init: Couldn't contact scanner on port "
-				"%s. Maybe it's not a scanner?\n",
+				"%s. Probably no scanner there?\n",
 				s_tmp->params.port->name);
+			ieee1284_release(s_tmp->params.port);
 			ieee1284_close(s_tmp->params.port);
 			s_tmp->scanner_present = SANE_FALSE;
 			continue;
 		}
 
+		/* put it back to sleep until we're ready to 
+		 * open for business again - this will only work
+		 * if we actually have a scanner there! */
+		DBG(100, "sane_init: And back to sleep again\n");
+		sanei_canon_pp_sleep_scanner(s_tmp->params.port);
+
+		/* leave the port open but not claimed - this is regardless 
+		 * of the return value of initialise */
+		ieee1284_release(s_tmp->params.port);
+
 		/* Finally, we're sure there's a scanner there! Now we
 		 * just have to load the weights file...*/
+
 		if (fix_weights_file(s_tmp) != SANE_STATUS_GOOD) {
 			DBG(1, "sane_init: Eeek! fix_weights_file failed for "
 				"scanner on port %s!\n", 
 				s_tmp->params.port->name);
-
-			ieee1284_close(s_tmp->params.port);
-			continue;
+			/* non-fatal.. scans will look ugly as sin unless
+			 * they calibrate */
 		}
 
 		/* Cocked, locked and ready to rock */
-
 		s_tmp->hw.model = s_tmp->params.name;
 		s_tmp->scanner_present = SANE_TRUE;
 	}
@@ -405,9 +455,9 @@ sane_get_devices (const SANE_Device ***dl, SANE_Bool local)
 		*dl = devlist;
 		return SANE_STATUS_GOOD;
 	}
-	devlist = malloc ((num_devices + 1) * sizeof (devlist[0]));
+	devlist = malloc((num_devices + 1) * sizeof(*devlist));
 	if (devlist == NULL)
-		return (SANE_STATUS_NO_MEM);
+		return SANE_STATUS_NO_MEM;
 
 	i = 0;
 	for (dev = first_dev; dev != NULL; dev = dev->next)
@@ -523,7 +573,7 @@ sane_open (SANE_String_Const name, SANE_Handle *h)
 	/* I put the scanner to sleep before, better wake it back up */
 
 	DBG(2, "sane_open: >> initialise\n");
-	tmp = sanei_canon_pp_initialise(&(cs->params));
+	tmp = sanei_canon_pp_initialise(&(cs->params), cs->init_mode);
 	DBG(2, "sane_open: << %d initialise\n", tmp);
 	if (tmp != 0) {
 		DBG(1, "sane_open: initialise returned %d, something is "
@@ -570,41 +620,39 @@ sane_open (SANE_String_Const name, SANE_Handle *h)
 	/* Resolution - determined by magic number */
 
 	if (cs->params.scanheadwidth == 2552)
-		cs->opt[OPT_RESOLUTION].constraint.word_list = res330;
+		cs->opt[OPT_RESOLUTION].constraint.word_list = res300;
 	else
-		cs->opt[OPT_RESOLUTION].constraint.word_list = res630;
+		cs->opt[OPT_RESOLUTION].constraint.word_list = res600;
 
 
 	/* TL-X */
-	if(!(tmp_range = (SANE_Range *)malloc(sizeof(SANE_Range))))
+	if(!(tmp_range = malloc(sizeof(*tmp_range))))
 		return SANE_STATUS_NO_MEM;
 	(*tmp_range).min = 0;
-	/* max is set later */
+	(*tmp_range).max = 215;
 	cs->opt[OPT_TL_X].constraint.range = tmp_range;
 
 	/* TL-Y */
-	if(!(tmp_range = (SANE_Range *)malloc(sizeof(SANE_Range))))
+	if(!(tmp_range = malloc(sizeof(*tmp_range))))
 		return SANE_STATUS_NO_MEM;
 	(*tmp_range).min = 0;
-	/* max is set later */
+	(*tmp_range).max = 296;
 	cs->opt[OPT_TL_Y].constraint.range = tmp_range;
 
 	/* BR-X */
-	if(!(tmp_range = (SANE_Range *)malloc(sizeof(SANE_Range))))
+	if(!(tmp_range = malloc(sizeof(*tmp_range))))
 		return SANE_STATUS_NO_MEM;
-	(*tmp_range).min = 64;
-	/* max is set later */
+	(*tmp_range).min = 3;
+	(*tmp_range).max = 216;
 	cs->opt[OPT_BR_X].constraint.range = tmp_range;
 
 	/* BR-Y */
-	if(!(tmp_range = (SANE_Range *)malloc(sizeof(SANE_Range))))
+	if(!(tmp_range = malloc(sizeof(*tmp_range))))
 		return SANE_STATUS_NO_MEM;
 	(*tmp_range).min = 1;
-	/* max is set later */
+	(*tmp_range).max = 297;
 	cs->opt[OPT_BR_Y].constraint.range = tmp_range;
 
-	/* set the max for all those ranges */
-	update_ranges(cs);
 
 	cs->opened = SANE_TRUE;
 	cs->setup = SANE_TRUE;
@@ -715,7 +763,7 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 							depths[cs->vals[opt]]);
 					break;
 				case OPT_RESOLUTION:
-					*((int *)val) = res630[cs->vals[opt]];
+					*((int *)val) = res600[cs->vals[opt]];
 					break;
 				default:
 					*((int *)val) = cs->vals[opt];
@@ -737,23 +785,13 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 						constraint.word_list[0];
 
 					while ((cs->vals[opt] <= maxresi) && 
-							(res630[cs->vals[opt]]
+							(res600[cs->vals[opt]]
 							 < *((int *)val)))
 					{
 						cs->vals[opt] += 1;
 					}
-					if (cs->vals[opt] != i)
-					{
-						update_ranges(cs);
-						/* shouldn't have to reload
-						 * params too, but xsane
-						 * seems a bit buggy in that
-						 * respect.  it's ok for us
-						 * not to change them */
-						if (info != NULL) *info |= 
-							SANE_INFO_RELOAD_OPTIONS | SANE_INFO_RELOAD_PARAMS;
-					}
-					if (res630[cs->vals[opt]] != 
+
+					if (res600[cs->vals[opt]] != 
 							*((int *)val))
 					{
 						if (info != NULL) *info |= 
@@ -784,23 +822,6 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 					break;
 				case OPT_TL_X:
 				case OPT_BR_X:
-					if ((i<cs->opt[opt].constraint.range->min) || (i>cs->opt[opt].constraint.range->max))
-						return SANE_STATUS_INVAL;
-					/* Rnd up to the next multiple of 4 */
-					if (i % 4) {
-						DBG(100, "Inexact: X requested"
-								": %d,",i);
-						if (opt == OPT_TL_X)
-							i -= (i % 4);
-						else
-							i += 4 - (i % 4);
-
-						DBG(100, " X provided: %d\n",i);
-						if (info != NULL) *info |= 
-							SANE_INFO_INEXACT; 
-					}
-					cs->vals[opt] = i;
-					break;
 				case OPT_TL_Y:
 				case OPT_BR_Y:
 					if ((i<cs->opt[opt].constraint.range->min) || (i>cs->opt[opt].constraint.range->max))
@@ -881,8 +902,8 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 	SANE_Status
 sane_get_parameters (SANE_Handle h, SANE_Parameters *params)
 {
-	CANONP_Scanner *cs = ((CANONP_Scanner *)h);
-	SANE_Int total_lines = 0;
+	int res, max_width, max_height, max_res;
+        CANONP_Scanner *cs = ((CANONP_Scanner *)h);
 	DBG(2, ">> sane_get_parameters (h=%p, params=%p)\n", h, params);
 
 	if (h == NULL) return SANE_STATUS_INVAL;
@@ -894,11 +915,41 @@ sane_get_parameters (SANE_Handle h, SANE_Parameters *params)
 		return SANE_STATUS_INVAL;
 	}
 
+	/* We use 600 res list here because the 300 res list is just a shorter
+	 * version, so this will always work. */
+	res = res600[cs->vals[OPT_RESOLUTION]];
+
 	/* 
 	 * These don't change whether we're scanning or not 
-	 * NOTE: Assumes options don't change after scanning commences
+	 * NOTE: Assumes options don't change after scanning commences, which
+         *       is part of the standard
 	 */
-	total_lines = cs->vals[OPT_BR_Y] - cs->vals[OPT_TL_Y];
+
+	/* Copy the options stored in the vals into the scaninfo */
+	params->pixels_per_line = 
+                ((cs->vals[OPT_BR_X] - cs->vals[OPT_TL_X]) * res) / MM_PER_IN;
+	params->lines = ((cs->vals[OPT_BR_Y] - cs->vals[OPT_TL_Y]) * res) 
+                / MM_PER_IN;
+
+	/* FIXME: Magic numbers ahead! */
+
+	max_res = cs->params.scanheadwidth == 2552 ? 300 : 600;
+
+	/* x values have to be divisible by 4 (round down) */
+	params->pixels_per_line -= (params->pixels_per_line%4);
+
+        /* Can't scan less than 64 */
+        if (params->pixels_per_line < 64) params->pixels_per_line = 64;
+
+	max_width = cs->params.scanheadwidth / (max_res / res);
+
+	max_height = (cs->params.scanheadwidth == 2552 ? 3508 : 7016) / 
+                                        (max_res / res);
+
+        if(params->pixels_per_line > max_width) 
+                params->pixels_per_line = max_width;
+        if(params->lines > max_height) params->lines = max_height;
+
 
 	params->depth = cs->vals[OPT_DEPTH] ? 16 : 8;
 
@@ -915,34 +966,24 @@ sane_get_parameters (SANE_Handle h, SANE_Parameters *params)
 			break;
 	}
 
-	params->pixels_per_line = cs->vals[OPT_BR_X] - cs->vals[OPT_TL_X];
 
 	if (!(params->pixels_per_line)) {
 		params->last_frame = SANE_TRUE;
 		params->lines = 0;
 	} 
 
-	/* Always assume next packet will be the last 
-	 * - frontends seem to like it that way */
+	/* Always the "last frame" */
 	params->last_frame = SANE_TRUE;
-	params->lines = total_lines - cs->lines_scanned;
 
-	switch (cs->vals[OPT_COLOUR_MODE]) 
-	{
-		case 0: /* Grey */
-			params->bytes_per_line = 
-				params->pixels_per_line * (params->depth/8);
-			break;
-		case 1: /* Colour */
-			params->bytes_per_line = 
-				params->pixels_per_line * (params->depth/8) * 3;
-			break;
-		default:
-			/* shouldn't happen */
-			break;
-	}
+	params->bytes_per_line = params->pixels_per_line * (params->depth/8) *
+		(cs->vals[OPT_COLOUR_MODE] ? 3 : 1);
 
-	/* FIXME: Do we need to account for the scanner's max buffer here? */
+	DBG(10, "get_params: bytes_per_line=%d, pixels_per_line=%d, lines=%d\n"
+                "max_res=%d, res=%d, max_height=%d, br_y=%d, tl_y=%d, "
+		"mm_per_in=%f\n",
+                params->bytes_per_line, params->pixels_per_line, params->lines,
+                max_res, res, max_height, cs->vals[OPT_BR_Y], 
+		cs->vals[OPT_TL_Y], MM_PER_IN);
 
 	DBG(2, "<< sane_get_parameters\n");
 	return SANE_STATUS_GOOD;
@@ -959,7 +1000,7 @@ sane_get_parameters (SANE_Handle h, SANE_Parameters *params)
 	SANE_Status
 sane_start (SANE_Handle h)
 {
-	int i,tmp;
+	unsigned int i, res, max_width, max_height, max_res, tmp;
 	CANONP_Scanner *cs = ((CANONP_Scanner *)h);
 	DBG(2, ">> sane_start (h=%p)\n", h);
 
@@ -973,30 +1014,49 @@ sane_start (SANE_Handle h)
 		return SANE_STATUS_INVAL;
 	}
 
-	/* Copy the options stored in the vals into the scaninfo */
-	cs->scan.width = cs->vals[OPT_BR_X] - cs->vals[OPT_TL_X];
-	cs->scan.height = cs->vals[OPT_BR_Y] - cs->vals[OPT_TL_Y];
-	cs->scan.xoffset = cs->vals[OPT_TL_X];
-	cs->scan.yoffset = cs->vals[OPT_TL_Y];
 
-	if (((cs->vals[OPT_BR_Y] - cs->vals[OPT_TL_Y]) <= 0) || 
-			((cs->vals[OPT_BR_X] - cs->vals[OPT_TL_X]) <= 0))
-	{
-		DBG(1,"sane_start: height = %d, Width = %d. "
-				"Can't scan void range!",
-				cs->scan.height, cs->scan.width);
-		return SANE_STATUS_INVAL;
-	}
-
-	/* We use 630 res list here because the 330 res list is just a shorter
+	/* We use 600 res list here because the 300 res list is just a shorter
 	 * version, so this will always work. */
-	tmp = res630[cs->vals[OPT_RESOLUTION]];
+	res = res600[cs->vals[OPT_RESOLUTION]];
 
-	DBG(10, "sane_start: val=%d, tmp=%d\n", 
-			cs->vals[OPT_RESOLUTION], tmp);
+	/* Copy the options stored in the vals into the scaninfo */
+	cs->scan.width = ((cs->vals[OPT_BR_X] - cs->vals[OPT_TL_X]) * res) 
+                / MM_PER_IN;
+	cs->scan.height = ((cs->vals[OPT_BR_Y] - cs->vals[OPT_TL_Y]) * res) 
+                / MM_PER_IN;
+
+	cs->scan.xoffset = (cs->vals[OPT_TL_X] * res) / MM_PER_IN;
+	cs->scan.yoffset = (cs->vals[OPT_TL_Y] * res) / MM_PER_IN;
+
+        /* 
+         * These values have to pass the requirements of not exceeding 
+         * dimensions (simple clipping) and both width values have to be some 
+         * integer multiple of 4 
+         */
+
+	/* FIXME: Magic numbers ahead! */
+
+	max_res = cs->params.scanheadwidth == 2552 ? 300 : 600;
+
+	/* x values have to be divisible by 4 (round down) */
+	cs->scan.width -= (cs->scan.width%4);
+	cs->scan.xoffset -= (cs->scan.xoffset%4);
+
+	/* Can't scan less than 64 */
+        if (cs->scan.width < 64) cs->scan.width = 64;
+
+	max_width = cs->params.scanheadwidth / (max_res / res);
+
+	max_height = (cs->params.scanheadwidth == 2552 ? 3508 : 7016) / 
+                                        (max_res / res);
+
+        if (cs->scan.width > max_width) cs->scan.width = max_width;
+	if (cs->scan.width + cs->scan.xoffset > max_width) cs->scan.xoffset = 
+		max_width - cs->scan.width;
+        if (cs->scan.height > max_height) cs->scan.height = max_height;
 
 	/* We pass a value to init_scan which is the power of 2 that 75
-	 * is multiplied for the resolution.  ie:
+	 * is multiplied by for the resolution.  ie:
 	 * 75 -> 0
 	 * 150 -> 1
 	 * 300 -> 2
@@ -1007,16 +1067,25 @@ sane_start (SANE_Handle h)
 	 */
 
 	i = 0;
-	while (tmp > 75)
+	while (res > 75)
 	{
 		i++;
-		tmp = tmp >> 1;
+		res = res >> 1;
 	}
 
 	/* FIXME? xres == yres for now. */
 	cs->scan.xresolution = i;
 	cs->scan.yresolution = i;
 
+	if (((cs->vals[OPT_BR_Y] - cs->vals[OPT_TL_Y]) <= 0) || 
+			((cs->vals[OPT_BR_X] - cs->vals[OPT_TL_X]) <= 0))
+	{
+		DBG(1,"sane_start: height = %d, Width = %d. "
+				"Can't scan void range!",
+				cs->scan.height, cs->scan.width);
+		return SANE_STATUS_INVAL;
+	}
+
 	cs->scan.mode = cs->vals[OPT_COLOUR_MODE];
 
 	DBG(10, ">> init_scan(%p, %p)\n", &(cs->params), &(cs->scan));
@@ -1030,22 +1099,11 @@ sane_start (SANE_Handle h)
 	cs->scanning = SANE_TRUE;
 	cs->cancelled = SANE_FALSE;
 	cs->sent_eof = SANE_FALSE;
-
-	/*  init_scan doesn't return a value yet... but Simon might get keen
-	*  one day 
-	if (!(init_scan(&(cs->params), &(cs->scan))))
-	{
-	cs->scanning = SANE_TRUE;
-	return SANE_STATUS_GOOD;
-	}
-	else
-	{
-	cs->scanning = SANE_FALSE;
-	return SANE_STATUS_IO_ERROR;
-	}
-	*/
+	cs->lines_scanned = 0;
+	cs->bytes_sent = 0;
 
 	DBG(2, "<< sane_start\n");
+
 	return SANE_STATUS_GOOD;
 }
 
@@ -1066,7 +1124,7 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 	unsigned int i;
 	short *shortptr;
 	SANE_Byte *charptr;
-	int max_buf, tmp;
+	int tmp;
 
 	static SANE_Byte *lbuf;
 	static unsigned int bytesleft;
@@ -1079,66 +1137,63 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 	if ((h == NULL) || (buf == NULL) || (lenp == NULL)) 
 	{
 		DBG(1, "sane_read: This frontend's passing me dodgy gear! "
-				"(h=%p, buf=%p, lenp=%p)\n",
-				h, buf, lenp);
+				"(h=%p, buf=%p, lenp=%p)\n", h, buf, lenp);
 		return SANE_STATUS_INVAL;
 	}
 
 	/* Now we have to see if we have some leftover from last time */
 
-	if (read_leftover != NULL) {
+	if (read_leftover != NULL) 
+	{
 		/* feed some more data in until we've run out - don't care 
 		 * whether or not we _think_ the scanner is scanning now, 
 		 * because we may still have data left over to send */
+		DBG(200, "sane_read: didn't send it all last time\n");
 
 		/* Now feed it some data from lbuf */
 		if (bytesleft <= (unsigned int)maxlen)
 		{
 			/* enough buffer to send the lot */
-			memcpy(buf, lbuf, bytesleft);
+			memcpy(buf, read_leftover, bytesleft);
 			free(lbuf);
 			*lenp = bytesleft;
 			lbuf = NULL;
 			read_leftover = NULL;
 			bytesleft = 0;
+                        cs->bytes_sent += bytesleft;
 			return SANE_STATUS_GOOD;
 
 		} else {
 			/* only enough to send maxlen */
-			memcpy(buf, lbuf, maxlen);
-			read_leftover = lbuf + maxlen;
+			memcpy(buf, read_leftover, maxlen);
+			read_leftover += maxlen;
 			bytesleft -= maxlen;
 			*lenp = maxlen;
+                        cs->bytes_sent += maxlen;
+		        DBG(100, "sane_read: sent %d bytes, still have %d to "
+                                "go\n", maxlen, bytesleft);
 			return SANE_STATUS_GOOD;
 		}
 
-	} else {
-		/* fix potential memory leak if cancelling at the very end
-		 * of a scan */
-		if (lbuf != NULL) free(lbuf);
-		lbuf = NULL;
-	}
+	} 
 
-	/* Has the last scan ended? */
-	if (((unsigned)cs->scan.height == (unsigned)cs->lines_scanned) 
-		|| !(cs->scanning))
+
+	/* Has the last scan ended (other than by cancelling)? */
+	if (((unsigned)cs->scan.height <= (unsigned)cs->lines_scanned) 
+	    || (cs->sent_eof) || !(cs->scanning))
 	{
-		if (cs->cancelled)
-			return SANE_STATUS_CANCELLED;
-
-		if (cs->sent_eof)
-		{
-			/* It's over mate, get over it */
-			DBG(10, "sane_read: Already sent EOF!\n");
-			return SANE_STATUS_INVAL;
-		} else {
-			cs->sent_eof = SANE_TRUE;
-			return SANE_STATUS_EOF;
-		}
+		cs->sent_eof = SANE_TRUE;
+		cs->scanning = SANE_FALSE;
+		cs->cancelled = SANE_FALSE;
+		cs->lines_scanned = 0;
+		cs->bytes_sent = 0;
+		read_leftover = NULL;
+		return SANE_STATUS_EOF;
 	}
 
-	/* At this point we have to read more data from the scanner */
-
+	/* At this point we have to read more data from the scanner - or the 
+	 * scan has been cancelled, which means we have to call read_segment
+	 * to leave the scanner consistant */
 
 	/* Decide how many lines we can fit into this buffer */
 	if (cs->vals[OPT_DEPTH] == 0)
@@ -1146,26 +1201,18 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 	else
 		bpl = cs->scan.width * (cs->vals[OPT_COLOUR_MODE] ? 6 : 2);
 
-	/* We will have as many lines as possible, or the size of the scan.
-	 * Each pixel is 5/4 bytes per colour, so we multiply the buffer 
-	 * size by 4/5 to get the number of lines. */
-	if ((int)maxlen > (BUF_MAX * 4 / 5))
-		max_buf = (BUF_MAX * 4 / 5);
-	else
-		max_buf = (int)maxlen;
+        /* New way: scan a whole scanner buffer full, and return as much as 
+         * the frontend wants.  It's faster and more reliable since the 
+         * scanners crack the shits if we ask for too many small packets */
+	lines = (BUF_MAX * 4 / 5) / bpl;
 
-	lines = max_buf / (int)bpl;
-	if (lines > cs->scan.height) lines = cs->scan.height;
-	if ((cs->scan.height - cs->lines_scanned) < lines) 
+	if (lines > (cs->scan.height - cs->lines_scanned)) 
 		lines = cs->scan.height - cs->lines_scanned;
 
 	if (!lines)
 	{
-		/* can't fit a whole line into the buffer */
-		DBG(10, "sane_read: Tighter than a duck's butt (can't fit a "
-				"whole scanline in the buffer)\n");
-		/* return SANE_STATUS_INVAL; */
-		/* Try and deal with this gracefully */
+		/* can't fit a whole line into the buffer 
+                 * (should never happen!) */
 		lines = 1;
 	}
 
@@ -1191,21 +1238,30 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 			(cs->params.id_string)+8);
 	DBG(10, "scan_params->: width=%d, height=%d, xoffset=%d, "
 			"yoffset=%d\n\txresolution=%d, yresolution=%d, "
-			"mode=%d\n\n",
+			"mode=%d, (lines=%d)\n",
 			cs->scan.width, cs->scan.height, 
 			cs->scan.xoffset, cs->scan.yoffset,
 			cs->scan.xresolution, cs->scan.yresolution,
-			cs->scan.mode
-	   );
-	DBG(10, "lines=%d\n",lines);
+			cs->scan.mode, lines);
 
-	DBG(2, ">> read_segment(%p, %p, %p, %d)\n",
-			&is, &(cs->params), &(cs->scan), lines);
-	tmp = sanei_canon_pp_read_segment(&is, &(cs->params), 
-			&(cs->scan), lines, cs->cal_valid);
+	DBG(2, ">> read_segment(%p, %p, %p, %d, %d, %d)\n",
+			&is, &(cs->params), &(cs->scan), lines,
+			cs->cal_valid, cs->scan.height - cs->lines_scanned);
+	tmp = sanei_canon_pp_read_segment(&is, &(cs->params), &(cs->scan), 
+			lines, cs->cal_valid, 
+			cs->scan.height - cs->lines_scanned);
 	DBG(2, "<< %d read_segment\n", tmp);
 
 	if (tmp != 0) {
+		if (cs->cancelled)
+		{
+			DBG(10, "sane_read: cancelling.\n");
+			cs->sent_eof = SANE_TRUE;
+			cs->scanning = SANE_FALSE;
+			read_leftover = NULL;
+			sanei_canon_pp_abort_scan(&(cs->params));
+			return SANE_STATUS_CANCELLED;
+		}
 		DBG(1, "sane_read: WARNING: read_segment returned %d!\n", tmp);
 		return SANE_STATUS_IO_ERROR;
 	}
@@ -1215,66 +1271,42 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 	cs->lines_scanned += lines;
 
 	/* translate data out of buffer */
-	if (cs->vals[OPT_COLOUR_MODE] == 0)
+	if (cs->vals[OPT_DEPTH] == 0)
 	{
-		/* fudge because simon's code is slack and returns greyscale as
-		 * RGB data, just with the same stuff in R,G,B. */
-		DBG(10, "sane_read: Initialising FUDGE engine 1.0\n");
-		if (cs->vals[OPT_DEPTH] == 0)
+		/* 8bpp */
+		for(i = 0; i < bytes; i++)
 		{
-			/* 8bpp */
-			for(i = 0; i < bytes * 3; i += 3)
+			charptr = lbuf + i;
+			if (cs->vals[OPT_COLOUR_MODE])
 			{
-				*((char *)lbuf + i/3) = 
-					*((char *)(is->image_data) + (i*2));
-			}
-		}
-		else
-		{
-			/* 16bpp */
-			for(i = 0; i < (bytes/2) * 3; i += 3)
-			{
-				/* Convert bigendian data to the local system */
-				*((short *)lbuf + i/3) = MAKE_SHORT(
-						*((char *)(is->image_data) 
-							+ (i*2)),
-						*((char *)(is->image_data) 
-							+ (i*2)+1)
-						);
+				if (i % 3 == 0) charptr += 2;
+				if (i % 3 == 2) charptr -= 2;
 			}
+			*charptr = *((char *)(is->image_data) + (i*2));
 		}
 	}
 	else
 	{
-		if (cs->vals[OPT_DEPTH] == 0)
+		/* 16bpp */
+		for(i = 0; i < (bytes/2); i++)
 		{
-			/* 8bpp */
-			for(i = 0; i < bytes; i++)
+			shortptr = ((short *)lbuf + i);
+			if (cs->vals[OPT_COLOUR_MODE])
 			{
-				charptr = lbuf + i;
-				if (i % 3 == 0) charptr += 2;
-				if (i % 3 == 2) charptr -= 2;
-				*(charptr) = 
-					*((char *)(is->image_data) + (i*2));
-			}
-		}
-		else
-		{
-			/* 16bpp */
-			for(i = 0; i < (bytes/2); i++)
-			{
-				shortptr = ((short *)lbuf + i);
 				if (i % 3 == 0) shortptr += 2;
 				if (i % 3 == 2) shortptr -= 2;
-				*shortptr = MAKE_SHORT(
-						*((char *)(is->image_data) +
-							(i*2)),
-						*((char *)(is->image_data) +
-							(i*2)+1));
 			}
+			*shortptr = MAKE_SHORT(
+					*((char *)(is->image_data) + (i*2)),
+					*((char *)(is->image_data) + (i*2)+1)
+					);
 		}
 	}
 
+	/* Free data structures allocated in read_segment */
+	free(is->image_data);
+	free(is);
+
 	/* Now feed it some data from lbuf */
 	if (bytes <= (unsigned int)maxlen)
 	{
@@ -1285,6 +1317,7 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 		lbuf = NULL;
 		read_leftover = NULL;
 		bytesleft = 0;
+                cs->bytes_sent += bytes;
 
 	} else {
 		/* only enough to send maxlen */
@@ -1292,6 +1325,9 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 		*lenp = maxlen;
 		read_leftover = lbuf + maxlen;
 		bytesleft = bytes - maxlen;
+                cs->bytes_sent += maxlen;
+		DBG(100, "sane_read: sent %d bytes, still have %d to go\n",
+                        maxlen, bytesleft);
 	}
 
 	if ((unsigned)cs->lines_scanned >= cs->scan.height)
@@ -1301,6 +1337,7 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 		DBG(10, "sane_read: Scan is finished.\n");
 		cs->scanning = SANE_FALSE;
 		cs->lines_scanned = 0;
+		cs->bytes_sent = 0;
 	}
 
 	DBG(2, "<< sane_read\n");
@@ -1318,30 +1355,22 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 	void
 sane_cancel (SANE_Handle h)
 {
-	int tmp;
 	/* Note: assume handle is valid apart from NULLs */
 	CANONP_Scanner *cs = ((CANONP_Scanner *)h);
+
 	DBG(2, ">> sane_cancel (h=%p)\n", h);
 	if (h == NULL) return;
 
-	if (cs->scanning == SANE_FALSE) 
+	read_leftover = NULL;
+
+	if (!(cs->scanning)) 
 	{
-		/* ensure we don't try to send old data */
-		read_leftover = NULL;
+		DBG(2, "<< sane_cancel (not scanning)\n");
 		return;
 	}
 
-	cs->scanning = SANE_FALSE;
-	/* cs->sent_eof = SANE_TRUE; */
 	cs->cancelled = SANE_TRUE;
-
-	cs->lines_scanned = 0;
-	DBG(2, "sane_cancel: >> abort_scan\n");
-	tmp = sanei_canon_pp_abort_scan(&(cs->params), &(cs->scan));
-	DBG(2, "sane_cancel: << abort_scan\n");
-	if (tmp != 0) {
-		DBG(1, "sane_cancel: WARNING: abort_scan returned %d!", tmp);
-	}
+	cs->params.abort_now = 1;
 
 	DBG(2, "<< sane_cancel\n");
 }
@@ -1437,12 +1466,9 @@ sane_exit (void)
 
 	first_dev = NULL;
 
-	/* FIXEDME: this creates a segfault in DLL code. */
-	/* freeing something it shouldn't perhaps? */
-	/* Turns out to be a bug in libieee1284 0.1.4 and only occurs on 2.2 
-	 * kernels - have sent mail to Tim Waugh, will leave commented out
-	 * until I receive a reply */
-	/* ieee1284_free_ports(&pl); */
+	/* FIXEDME: this created a segfault in DLL code. */
+	/* Bug was fixed in libieee1284 0.1.5 */
+	ieee1284_free_ports(&pl);
 
 	DBG(2, "<< sane_exit\n");
 }
@@ -1517,15 +1543,15 @@ static SANE_Status init_device(struct parport *pp)
 
 	DBG(2, ">> init_device\n");
 
-	cs = malloc (sizeof (CANONP_Scanner));
+	cs = malloc(sizeof(*cs));
 	if (cs == NULL)
 	{
 		return SANE_STATUS_NO_MEM;
 	}
-	memset (cs, 0, sizeof (CANONP_Scanner));
+	memset(cs, 0, sizeof(*cs));
 
 #if 0
-	if ((cs->params.port = malloc(sizeof(*pp))) == NULL) 
+	if ((cs->params.port = malloc(sizeof(*(cs->params.port)))) == NULL) 
 		return SANE_STATUS_NO_MEM;
 
 	memcpy(cs->params.port, pp, sizeof(*pp));
@@ -1549,6 +1575,9 @@ static SANE_Status init_device(struct parport *pp)
 	cs->scanning = SANE_FALSE;
 	cs->cancelled = SANE_FALSE;
 	cs->sent_eof = SANE_TRUE;
+	cs->lines_scanned = 0;
+	cs->bytes_sent = 0;
+	cs->init_mode = INITMODE_AUTO;
 
 	DBG(10, "init_device: [configuring options]\n");
 
@@ -1599,19 +1628,19 @@ static SANE_Status init_device(struct parport *pp)
 	DBG(100, "init_device: configuring opt: tl-x\n");
 
 	/* The top-left-x */
-	cs->opt[OPT_TL_X].unit = SANE_UNIT_PIXEL;
+	cs->opt[OPT_TL_X].unit = SANE_UNIT_MM;
 	cs->opt[OPT_TL_X].constraint_type = SANE_CONSTRAINT_RANGE;
 
 	DBG(100, "init_device: configuring opt: tl-y\n");
 
 	/* The top-left-y */
-	cs->opt[OPT_TL_Y].unit = SANE_UNIT_PIXEL;
+	cs->opt[OPT_TL_Y].unit = SANE_UNIT_MM;
 	cs->opt[OPT_TL_Y].constraint_type = SANE_CONSTRAINT_RANGE;
 
 	DBG(100, "init_device: configuring opt: br-x\n");
 
 	/* The bottom-right-x */
-	cs->opt[OPT_BR_X].unit = SANE_UNIT_PIXEL;
+	cs->opt[OPT_BR_X].unit = SANE_UNIT_MM;
 	cs->opt[OPT_BR_X].constraint_type = SANE_CONSTRAINT_RANGE;
 	/* default scan width */
 	cs->vals[OPT_BR_X] = 100;
@@ -1619,7 +1648,7 @@ static SANE_Status init_device(struct parport *pp)
 	DBG(100, "init_device: configuring opt: br-y\n");
 
 	/* The bottom-right-y */
-	cs->opt[OPT_BR_Y].unit = SANE_UNIT_PIXEL;
+	cs->opt[OPT_BR_Y].unit = SANE_UNIT_MM;
 	cs->opt[OPT_BR_Y].constraint_type = SANE_CONSTRAINT_RANGE;
 	cs->vals[OPT_BR_Y] = 100;
 
@@ -1683,50 +1712,6 @@ sane_get_select_fd (SANE_Handle h, SANE_Int *fdp)
 	return SANE_STATUS_UNSUPPORTED;
 }
 
-/*************************************************************************
- *
- * update_ranges(): when the res is changed, the max scan size in pixels 
- * has to be changed.
- *
- *************************************************************************/
-static void update_ranges(CANONP_Scanner *cs) 
-{
-	int new_width, new_height, max_res;
-
-	/* FIXME: Magic numbers ahead! */
-
-	max_res = cs->params.scanheadwidth == 2552 ? 300 : 600;
-
-	new_width = cs->params.scanheadwidth / 
-		(max_res / res630[cs->vals[OPT_RESOLUTION]]);
-	new_height = (cs->params.scanheadwidth == 2552 ? 3508 : 7016) /
-		(max_res / res630[cs->vals[OPT_RESOLUTION]]);
-
-	/* Max width has to be divisible by 4 */
-	new_width -= (new_width%4);
-
-	DBG(10, "update_ranges: new_width = %d, new_height = %d\n",
-			new_width, new_height);
-
-	(*((SANE_Range *)(cs->opt[OPT_TL_X].constraint.range))).max = 
-		new_width - 1;
-	(*((SANE_Range *)(cs->opt[OPT_TL_Y].constraint.range))).max = 
-		new_height - 1;
-	(*((SANE_Range *)(cs->opt[OPT_BR_X].constraint.range))).max = 
-		new_width;
-	(*((SANE_Range *)(cs->opt[OPT_BR_Y].constraint.range))).max = 
-		new_height;
-
-	if (cs->vals[OPT_TL_X] > cs->opt[OPT_TL_X].constraint.range->max)
-		cs->vals[OPT_TL_X] = cs->opt[OPT_TL_X].constraint.range->max;
-	if (cs->vals[OPT_BR_X] > cs->opt[OPT_BR_X].constraint.range->max)
-		cs->vals[OPT_BR_X] = cs->opt[OPT_BR_X].constraint.range->max;
-	if (cs->vals[OPT_TL_Y] > cs->opt[OPT_TL_Y].constraint.range->max)
-		cs->vals[OPT_TL_Y] = cs->opt[OPT_TL_Y].constraint.range->max;
-	if (cs->vals[OPT_BR_Y] > cs->opt[OPT_BR_Y].constraint.range->max)
-		cs->vals[OPT_BR_Y] = cs->opt[OPT_BR_Y].constraint.range->max;
-}
-
 
 /*************************************************************************
  *
@@ -1821,7 +1806,7 @@ static SANE_Status fix_weights_file(CANONP_Scanner *cs)
 		cs->weights_file = tmp;
 	}
 
-	if ((f_stat = malloc(sizeof(struct stat))) == NULL)
+	if ((f_stat = malloc(sizeof(*f_stat))) == NULL)
 		return SANE_STATUS_NO_MEM;
 
 	if(stat(cs->weights_file, f_stat))
@@ -1841,8 +1826,6 @@ static SANE_Status fix_weights_file(CANONP_Scanner *cs)
 	}
 	else
 	{
-		free(f_stat);
-		f_stat = NULL;
 
 		/* No error returned.. Check read/writability */
 		i = open(cs->weights_file, O_RDWR | O_APPEND);
@@ -1878,6 +1861,9 @@ static SANE_Status fix_weights_file(CANONP_Scanner *cs)
 		}
 	}
 
+	/* cleanup */
+	free(f_stat);
+
 	return SANE_STATUS_GOOD;
 }
 
@@ -1974,9 +1960,28 @@ SANE_Status detect_mode(CANONP_Scanner *cs)
 	}
 
 	/* Check to make sure ECP mode really is supported */
+	/* Have disabled the hardware ECP check because it's always supported 
+	 * by libieee1284 now, and it's too prone to hitting a ppdev bug 
+	 */
+
+	/* Disabled check entirely.. check now in initialise when we 
+	 * actually do a read */
+#if 0
 	if ((cs->ieee1284_mode == M1284_ECP) ||
 			(cs->ieee1284_mode == M1284_ECPSWE))
 	{
+		DBG(1, "detect_mode: attempting a 0 byte read, if we hang "
+				"here, it's a ppdev bug!\n");
+		/* 
+		 * 29/06/02 
+		 * NOTE:
+		 * This causes an infinite loop in ppdev on 2.4.18.  
+		 * Not checking on hardware ECP mode should work-around 
+		 * effectively.
+		 *
+		 * I have sent email to twaugh about it, should be fixed in 
+		 * 2.4.19 and above.
+		 */
 		if (ieee1284_ecp_read_data(cs->params.port, 0, NULL, 0) == 
 				E1284_NOTIMPL)
 		{
@@ -1986,6 +1991,7 @@ SANE_Status detect_mode(CANONP_Scanner *cs)
 			cs->ieee1284_mode = M1284_NIBBLE;
 		}
 	}
+#endif
 
 	if (force_nibble == SANE_TRUE) {
 		DBG(10, "detect_mode: Nibble mode force in effect.\n");
