@@ -79,6 +79,20 @@
  * Return number of elements of an array.
  *
  */
+
+/** @fn extern SANE_Status sanei_check_value (const SANE_Option_Descriptor * opt, void * value);
+ * Check the constraints of a SANE option.
+ *
+ * @param opt   option to check
+ * @param value value of the option
+ *
+ * @return 
+ * - SANE_STATUS_GOOD     - on success
+ * - SANE_STATUS_INVAL    - if the value doesn't fit inside the constraint
+ *   or any other error occured
+ * @sa sanei_constrain_value()
+ */
+
 /** @fn extern SANE_Status sanei_constrain_value (const SANE_Option_Descriptor * opt, void * value, SANE_Word * info);
  * Check the constraints of a SANE option and adjust its value if necessary.
  *
@@ -93,7 +107,9 @@
  * - SANE_STATUS_GOOD     - on success
  * - SANE_STATUS_INVAL    - if the function wasn't able to fit value into the
  *   constraint or any other error occured
+ * @sa sanei_check_value()
  */
+
 /* @} */
 
 /** @name Private macros
@@ -121,7 +137,11 @@
 /** @hideinitializer */
 #define PASTE(x,y)	PASTE1(x,y)
 
+extern SANE_Status sanei_check_value (const SANE_Option_Descriptor * opt,
+				      void * value);
+
 extern SANE_Status sanei_constrain_value (const SANE_Option_Descriptor * opt,
 					  void * value, SANE_Word * info);
 
+
 #endif /* sanei_h */
